#!/bin/sh
#
# Copyright (C) 2012 Univention GmbH
#
# http://www.univention.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.
# Copy this file to the initrd as /sbin/custom_partition normally with an custom ucc package.
# To test the script the following steps can be done:
#   initrd="ucc-1.0-thinclient-image.img.initrd"
#   mkdir -p /var/lib/univention-client-boot/initrd
#   cd /var/lib/univention-client-boot/initrd
#   cp ../${initrd} ../${initrd}.BACKUP
#   mv ../${initrd} ../${initrd}.gz
#   gunzip -c ../${initrd}.gz | cpio -id
#   rm ../${initrd}.gz
#   cp /usr/share/doc/ucc-image-toolkit/example/custom_partition.example sbin/custom_partition
#   chmod +x sbin/custom_partition
#   find . | cpio --create --format='newc' >../${initrd}
#   gzip ../${initrd}
#   mv ../${initrd}.gz ../${initrd}

debugsleep=3
debugheader="======= CUSTOM PARTITION: "

. /scripts/functions

test "$verbose" = "y" && set -x

# Get device
partition_device="$1"


# Load keyboard drivers
modprobe i8042
modprobe atkbd

# Ensure the console is prepared for the user interaction
run_scripts /scripts/panic

echo "$debugheader"   "start custom partition script"
sleep "$debugsleep"; 

# Create GPT
pt="$(/sbin/parted -s ${partition_device} unit MiB print | sed -ne 's|Partition Table: ||p')"
if [ "$pt" != "gpt" ]; then
    parted -s "${partition_device}" unit MiB mklabel GPT
fi

echo "$debugheader"   "checked GPT header"
sleep "$debugsleep"; 

echo "Delete partition: auto determination"

d_name="$(echo $partition_device | sed -e 's|/dev/||')"
for partition_path in $(ls -d /sys/block/${d_name}/${d_name}* 2>/dev/null); do
    partition="$(basename $partition_path)"
    partition_number="$(echo $partition | sed -e "s|${d_name}||")"
    parted -s "${partition_device}" unit MiB rm "$partition_number"
done

echo "$debugheader"   "removed old partitions"
sleep "$debugsleep"; 

disk_size=$(/sbin/parted -s "${partition_device}" unit MiB print | sed -ne "s|^Disk ${partition_device}: ||;s|MiB$||p")
type="primary"

# Create partitions
parted -s "${partition_device}" unit MiB mkpart "$type" "10" "60"
parted -s "${partition_device}" unit MiB set 1 bios_grub on
parted -s "${partition_device}" unit MiB mkpart "$type" "61" "561" # /boot
parted -s "${partition_device}" unit MiB mkpart "$type" "562" "$disk_size" # lvm

echo "$debugheader"   "created base partitions"
sleep "$debugsleep"; 

echo "$debugheader"   "going to create encrypted partition"
sleep "$debugsleep"; 

# Create luks container
echo -n "univention" | cryptsetup --key-file - luksFormat "${partition_device}3"
 
echo "$debugheader"   "cryptsetup luksformat finished"
sleep "$debugsleep"; 

# Open luks container
echo -n "univention" | cryptsetup --key-file - luksOpen "${partition_device}3" "crypt"

echo "$debugheader"   "cryptsetup luksopen finished"
sleep "$debugsleep"; 
 
# Create PV
lvm pvcreate /dev/mapper/crypt
 
# Create volume group
lvm vgcreate vg_ucc /dev/mapper/crypt

# Create logical volume for swap (GB Ram + 1 GB)
kbram="$(grep MemTotal /proc/meminfo | awk '{print $2}')"
lvm lvcreate -L$(expr $kbram / 1000 / 1000 + 1)G -n swap vg_ucc

echo "$debugheader"   "swap creation finished"
sleep "$debugsleep"; 

mkswap /dev/mapper/vg_ucc-swap
 
echo "$debugheader"   "mkswap finished"
sleep "$debugsleep"; 

# Create logical volume for /
lvm lvcreate -L25G -n rootfs vg_ucc

echo "$debugheader"   "lvm creation finished"
sleep "$debugsleep"; 

# Create root filesystem
mkfs.ext4 /dev/mapper/vg_ucc-rootfs

echo "$debugheader"   "mkfs rootfs finished"
sleep "$debugsleep"; 

# Get Free PE
## Example:
## Free PE / Size 8954 / 34.98 GB
# freepe="$(lvm vgdisplay | grep "Free  PE" | awk '{print $5}')"

# Create logical volume for /home
lvm lvcreate -l100%FREE -n home vg_ucc

# Create clbv filesystem
mkfs.ext4 /dev/mapper/vg_ucc-home
 
echo "$debugheader"   "mkfs home-fs finished"
sleep "$debugsleep"; 

# Create boot filesystem
/sbin/mkfs.ext4 "${partition_device}2"

echo "$debugheader"   "mkfs boot finished"
sleep "$debugsleep"; 

# Mount the root filesystem
mkdir -p /ucc_tmp/root
mount /dev/mapper/vg_ucc-rootfs /ucc_tmp/root

# Mark device as root device
touch /ucc_tmp/root/ucc_root_device

# Create fstab file
echo "/dev/loop0    /    ext4    errors=remount-ro    0    1" >>/ucc_tmp/fstab
echo "/dev/mapper/vg_ucc-swap    swap    swap    0    0" >>/ucc_tmp/fstab
echo "${partition_device}2    /boot    ext4    defaults    0    2" >>/ucc_tmp/fstab
echo "/dev/mapper/vg_ucc-home    /home    ext4    defaults    0    2" >>/ucc_tmp/fstab

# Copy files from the image to the boot partition
echo "${partition_device}2 ext4 /boot" >> /ucc_tmp/root/copy_files

# Create a script which sets set cryptopts in grub
cat >/ucc_tmp/root/customer.script <<__H1__
cat >/root/grub_append.sh <<'__H2__'
#!/bin/sh
ucr set grub/append="\$(ucr get grub/append) cryptopts=target=crypt,source=${partition_device}3,lvm=vg_ucc-rootfs resume=/dev/mapper/vg_ucc-swap"
__H2__
chmod +x /root/grub_append.sh
chroot root ./grub_append.sh
rm /root/grub_append.sh
__H1__


#echo "$debugheader"   "exit custom partition"
#sleep "$debugsleep"; 

return 0
